# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, Union, Iterable, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._streaming import Stream, AsyncStream
from ...types.chat import completion_create_params, completion_models_params
from ..._base_client import make_request_options
from ...types.chat.chat_completion_chunk import ChatCompletionChunk
from ...types.chat.completion_create_response import CompletionCreateResponse
from ...types.chat.completion_models_response import CompletionModelsResponse

__all__ = ["CompletionsResource", "AsyncCompletionsResource"]


class CompletionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return CompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return CompletionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse:
        """
        Chat Completions

        Args:
          messages: openai standard message format

          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          audio: Parameters for audio output. Required when audio output is requested with
              modalities: ['audio'].

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far.

          function_call: Deprecated in favor of tool_choice. Controls which function is called by the
              model.

          functions: Deprecated in favor of tools. A list of functions the model may generate JSON
              inputs for.

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Maps
              tokens to bias values from -100 to 100.

          logprobs: Whether to return log probabilities of the output tokens or not.

          max_completion_tokens: An upper bound for the number of tokens that can be generated, including visible
              output tokens and reasoning tokens.

          max_tokens: Deprecated in favor of max_completion_tokens. The maximum number of tokens to
              generate.

          metadata: Developer-defined tags and values used for filtering completions in the
              dashboard.

          modalities: Output types that you would like the model to generate for this request.

          n: How many chat completion choices to generate for each input message.

          parallel_tool_calls: Whether to enable parallel function calling during tool use.

          prediction: Static predicted output content, such as the content of a text file being
              regenerated.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize tokens based on whether
              they appear in the text so far.

          reasoning_effort: For o1 models only. Constrains effort on reasoning. Values: low, medium, high.

          response_format: An object specifying the format that the model must output.

          seed: If specified, system will attempt to sample deterministically for repeated
              requests with same seed.

          stop: Up to 4 sequences where the API will stop generating further tokens.

          store: Whether to store the output for use in model distillation or evals products.

          stream: If true, partial message deltas will be sent as server-sent events.

          stream_options: Options for streaming response. Only set this when stream is true.

          temperature: What sampling temperature to use. Higher values make output more random, lower
              more focused.

          tool_choice: Controls which tool is called by the model. Values: none, auto, required, or
              specific tool.

          tools: A list of tools the model may call. Currently, only functions are supported. Max
              128 functions.

          top_k: Only sample from the top K options for each subsequent token

          top_logprobs: Number of most likely tokens to return at each position, with associated log
              probability.

          top_p: Alternative to temperature. Only tokens comprising top_p probability mass are
              considered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        stream: Literal[True],
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[ChatCompletionChunk]:
        """
        Chat Completions

        Args:
          messages: openai standard message format

          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          stream: If true, partial message deltas will be sent as server-sent events.

          audio: Parameters for audio output. Required when audio output is requested with
              modalities: ['audio'].

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far.

          function_call: Deprecated in favor of tool_choice. Controls which function is called by the
              model.

          functions: Deprecated in favor of tools. A list of functions the model may generate JSON
              inputs for.

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Maps
              tokens to bias values from -100 to 100.

          logprobs: Whether to return log probabilities of the output tokens or not.

          max_completion_tokens: An upper bound for the number of tokens that can be generated, including visible
              output tokens and reasoning tokens.

          max_tokens: Deprecated in favor of max_completion_tokens. The maximum number of tokens to
              generate.

          metadata: Developer-defined tags and values used for filtering completions in the
              dashboard.

          modalities: Output types that you would like the model to generate for this request.

          n: How many chat completion choices to generate for each input message.

          parallel_tool_calls: Whether to enable parallel function calling during tool use.

          prediction: Static predicted output content, such as the content of a text file being
              regenerated.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize tokens based on whether
              they appear in the text so far.

          reasoning_effort: For o1 models only. Constrains effort on reasoning. Values: low, medium, high.

          response_format: An object specifying the format that the model must output.

          seed: If specified, system will attempt to sample deterministically for repeated
              requests with same seed.

          stop: Up to 4 sequences where the API will stop generating further tokens.

          store: Whether to store the output for use in model distillation or evals products.

          stream_options: Options for streaming response. Only set this when stream is true.

          temperature: What sampling temperature to use. Higher values make output more random, lower
              more focused.

          tool_choice: Controls which tool is called by the model. Values: none, auto, required, or
              specific tool.

          tools: A list of tools the model may call. Currently, only functions are supported. Max
              128 functions.

          top_k: Only sample from the top K options for each subsequent token

          top_logprobs: Number of most likely tokens to return at each position, with associated log
              probability.

          top_p: Alternative to temperature. Only tokens comprising top_p probability mass are
              considered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        stream: bool,
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | Stream[ChatCompletionChunk]:
        """
        Chat Completions

        Args:
          messages: openai standard message format

          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          stream: If true, partial message deltas will be sent as server-sent events.

          audio: Parameters for audio output. Required when audio output is requested with
              modalities: ['audio'].

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far.

          function_call: Deprecated in favor of tool_choice. Controls which function is called by the
              model.

          functions: Deprecated in favor of tools. A list of functions the model may generate JSON
              inputs for.

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Maps
              tokens to bias values from -100 to 100.

          logprobs: Whether to return log probabilities of the output tokens or not.

          max_completion_tokens: An upper bound for the number of tokens that can be generated, including visible
              output tokens and reasoning tokens.

          max_tokens: Deprecated in favor of max_completion_tokens. The maximum number of tokens to
              generate.

          metadata: Developer-defined tags and values used for filtering completions in the
              dashboard.

          modalities: Output types that you would like the model to generate for this request.

          n: How many chat completion choices to generate for each input message.

          parallel_tool_calls: Whether to enable parallel function calling during tool use.

          prediction: Static predicted output content, such as the content of a text file being
              regenerated.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize tokens based on whether
              they appear in the text so far.

          reasoning_effort: For o1 models only. Constrains effort on reasoning. Values: low, medium, high.

          response_format: An object specifying the format that the model must output.

          seed: If specified, system will attempt to sample deterministically for repeated
              requests with same seed.

          stop: Up to 4 sequences where the API will stop generating further tokens.

          store: Whether to store the output for use in model distillation or evals products.

          stream_options: Options for streaming response. Only set this when stream is true.

          temperature: What sampling temperature to use. Higher values make output more random, lower
              more focused.

          tool_choice: Controls which tool is called by the model. Values: none, auto, required, or
              specific tool.

          tools: A list of tools the model may call. Currently, only functions are supported. Max
              128 functions.

          top_k: Only sample from the top K options for each subsequent token

          top_logprobs: Number of most likely tokens to return at each position, with associated log
              probability.

          top_p: Alternative to temperature. Only tokens comprising top_p probability mass are
              considered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["messages", "model"], ["messages", "model", "stream"])
    def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | Stream[ChatCompletionChunk]:
        return self._post(
            "/v5/chat/completions",
            body=maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "audio": audio,
                    "frequency_penalty": frequency_penalty,
                    "function_call": function_call,
                    "functions": functions,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_completion_tokens": max_completion_tokens,
                    "max_tokens": max_tokens,
                    "metadata": metadata,
                    "modalities": modalities,
                    "n": n,
                    "parallel_tool_calls": parallel_tool_calls,
                    "prediction": prediction,
                    "presence_penalty": presence_penalty,
                    "reasoning_effort": reasoning_effort,
                    "response_format": response_format,
                    "seed": seed,
                    "stop": stop,
                    "store": store,
                    "stream": stream,
                    "stream_options": stream_options,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                },
                completion_create_params.CompletionCreateParamsStreaming
                if stream
                else completion_create_params.CompletionCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=cast(
                Any, CompletionCreateResponse
            ),  # Union types cannot be passed in as arguments in the type system
            stream=stream or False,
            stream_cls=Stream[ChatCompletionChunk],
        )

    def models(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        model_vendor: Literal[
            "openai",
            "cohere",
            "vertex_ai",
            "anthropic",
            "azure",
            "gemini",
            "launch",
            "llmengine",
            "model_zoo",
            "bedrock",
            "xai",
            "fireworks_ai",
        ]
        | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionModelsResponse:
        """
        List Chat Completion Models

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v5/chat/completions/models",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "model_vendor": model_vendor,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    completion_models_params.CompletionModelsParams,
                ),
            ),
            cast_to=CompletionModelsResponse,
        )


class AsyncCompletionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncCompletionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse:
        """
        Chat Completions

        Args:
          messages: openai standard message format

          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          audio: Parameters for audio output. Required when audio output is requested with
              modalities: ['audio'].

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far.

          function_call: Deprecated in favor of tool_choice. Controls which function is called by the
              model.

          functions: Deprecated in favor of tools. A list of functions the model may generate JSON
              inputs for.

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Maps
              tokens to bias values from -100 to 100.

          logprobs: Whether to return log probabilities of the output tokens or not.

          max_completion_tokens: An upper bound for the number of tokens that can be generated, including visible
              output tokens and reasoning tokens.

          max_tokens: Deprecated in favor of max_completion_tokens. The maximum number of tokens to
              generate.

          metadata: Developer-defined tags and values used for filtering completions in the
              dashboard.

          modalities: Output types that you would like the model to generate for this request.

          n: How many chat completion choices to generate for each input message.

          parallel_tool_calls: Whether to enable parallel function calling during tool use.

          prediction: Static predicted output content, such as the content of a text file being
              regenerated.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize tokens based on whether
              they appear in the text so far.

          reasoning_effort: For o1 models only. Constrains effort on reasoning. Values: low, medium, high.

          response_format: An object specifying the format that the model must output.

          seed: If specified, system will attempt to sample deterministically for repeated
              requests with same seed.

          stop: Up to 4 sequences where the API will stop generating further tokens.

          store: Whether to store the output for use in model distillation or evals products.

          stream: If true, partial message deltas will be sent as server-sent events.

          stream_options: Options for streaming response. Only set this when stream is true.

          temperature: What sampling temperature to use. Higher values make output more random, lower
              more focused.

          tool_choice: Controls which tool is called by the model. Values: none, auto, required, or
              specific tool.

          tools: A list of tools the model may call. Currently, only functions are supported. Max
              128 functions.

          top_k: Only sample from the top K options for each subsequent token

          top_logprobs: Number of most likely tokens to return at each position, with associated log
              probability.

          top_p: Alternative to temperature. Only tokens comprising top_p probability mass are
              considered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        stream: Literal[True],
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[ChatCompletionChunk]:
        """
        Chat Completions

        Args:
          messages: openai standard message format

          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          stream: If true, partial message deltas will be sent as server-sent events.

          audio: Parameters for audio output. Required when audio output is requested with
              modalities: ['audio'].

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far.

          function_call: Deprecated in favor of tool_choice. Controls which function is called by the
              model.

          functions: Deprecated in favor of tools. A list of functions the model may generate JSON
              inputs for.

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Maps
              tokens to bias values from -100 to 100.

          logprobs: Whether to return log probabilities of the output tokens or not.

          max_completion_tokens: An upper bound for the number of tokens that can be generated, including visible
              output tokens and reasoning tokens.

          max_tokens: Deprecated in favor of max_completion_tokens. The maximum number of tokens to
              generate.

          metadata: Developer-defined tags and values used for filtering completions in the
              dashboard.

          modalities: Output types that you would like the model to generate for this request.

          n: How many chat completion choices to generate for each input message.

          parallel_tool_calls: Whether to enable parallel function calling during tool use.

          prediction: Static predicted output content, such as the content of a text file being
              regenerated.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize tokens based on whether
              they appear in the text so far.

          reasoning_effort: For o1 models only. Constrains effort on reasoning. Values: low, medium, high.

          response_format: An object specifying the format that the model must output.

          seed: If specified, system will attempt to sample deterministically for repeated
              requests with same seed.

          stop: Up to 4 sequences where the API will stop generating further tokens.

          store: Whether to store the output for use in model distillation or evals products.

          stream_options: Options for streaming response. Only set this when stream is true.

          temperature: What sampling temperature to use. Higher values make output more random, lower
              more focused.

          tool_choice: Controls which tool is called by the model. Values: none, auto, required, or
              specific tool.

          tools: A list of tools the model may call. Currently, only functions are supported. Max
              128 functions.

          top_k: Only sample from the top K options for each subsequent token

          top_logprobs: Number of most likely tokens to return at each position, with associated log
              probability.

          top_p: Alternative to temperature. Only tokens comprising top_p probability mass are
              considered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        stream: bool,
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | AsyncStream[ChatCompletionChunk]:
        """
        Chat Completions

        Args:
          messages: openai standard message format

          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          stream: If true, partial message deltas will be sent as server-sent events.

          audio: Parameters for audio output. Required when audio output is requested with
              modalities: ['audio'].

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far.

          function_call: Deprecated in favor of tool_choice. Controls which function is called by the
              model.

          functions: Deprecated in favor of tools. A list of functions the model may generate JSON
              inputs for.

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Maps
              tokens to bias values from -100 to 100.

          logprobs: Whether to return log probabilities of the output tokens or not.

          max_completion_tokens: An upper bound for the number of tokens that can be generated, including visible
              output tokens and reasoning tokens.

          max_tokens: Deprecated in favor of max_completion_tokens. The maximum number of tokens to
              generate.

          metadata: Developer-defined tags and values used for filtering completions in the
              dashboard.

          modalities: Output types that you would like the model to generate for this request.

          n: How many chat completion choices to generate for each input message.

          parallel_tool_calls: Whether to enable parallel function calling during tool use.

          prediction: Static predicted output content, such as the content of a text file being
              regenerated.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize tokens based on whether
              they appear in the text so far.

          reasoning_effort: For o1 models only. Constrains effort on reasoning. Values: low, medium, high.

          response_format: An object specifying the format that the model must output.

          seed: If specified, system will attempt to sample deterministically for repeated
              requests with same seed.

          stop: Up to 4 sequences where the API will stop generating further tokens.

          store: Whether to store the output for use in model distillation or evals products.

          stream_options: Options for streaming response. Only set this when stream is true.

          temperature: What sampling temperature to use. Higher values make output more random, lower
              more focused.

          tool_choice: Controls which tool is called by the model. Values: none, auto, required, or
              specific tool.

          tools: A list of tools the model may call. Currently, only functions are supported. Max
              128 functions.

          top_k: Only sample from the top K options for each subsequent token

          top_logprobs: Number of most likely tokens to return at each position, with associated log
              probability.

          top_p: Alternative to temperature. Only tokens comprising top_p probability mass are
              considered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["messages", "model"], ["messages", "model", "stream"])
    async def create(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        model: str,
        audio: Dict[str, object] | Omit = omit,
        frequency_penalty: float | Omit = omit,
        function_call: Dict[str, object] | Omit = omit,
        functions: Iterable[Dict[str, object]] | Omit = omit,
        logit_bias: Dict[str, int] | Omit = omit,
        logprobs: bool | Omit = omit,
        max_completion_tokens: int | Omit = omit,
        max_tokens: int | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        modalities: SequenceNotStr[str] | Omit = omit,
        n: int | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        prediction: Dict[str, object] | Omit = omit,
        presence_penalty: float | Omit = omit,
        reasoning_effort: str | Omit = omit,
        response_format: Dict[str, object] | Omit = omit,
        seed: int | Omit = omit,
        stop: Union[str, SequenceNotStr[str]] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        stream_options: Dict[str, object] | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_k: int | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | AsyncStream[ChatCompletionChunk]:
        return await self._post(
            "/v5/chat/completions",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "audio": audio,
                    "frequency_penalty": frequency_penalty,
                    "function_call": function_call,
                    "functions": functions,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_completion_tokens": max_completion_tokens,
                    "max_tokens": max_tokens,
                    "metadata": metadata,
                    "modalities": modalities,
                    "n": n,
                    "parallel_tool_calls": parallel_tool_calls,
                    "prediction": prediction,
                    "presence_penalty": presence_penalty,
                    "reasoning_effort": reasoning_effort,
                    "response_format": response_format,
                    "seed": seed,
                    "stop": stop,
                    "store": store,
                    "stream": stream,
                    "stream_options": stream_options,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                },
                completion_create_params.CompletionCreateParamsStreaming
                if stream
                else completion_create_params.CompletionCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=cast(
                Any, CompletionCreateResponse
            ),  # Union types cannot be passed in as arguments in the type system
            stream=stream or False,
            stream_cls=AsyncStream[ChatCompletionChunk],
        )

    async def models(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        model_vendor: Literal[
            "openai",
            "cohere",
            "vertex_ai",
            "anthropic",
            "azure",
            "gemini",
            "launch",
            "llmengine",
            "model_zoo",
            "bedrock",
            "xai",
            "fireworks_ai",
        ]
        | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionModelsResponse:
        """
        List Chat Completion Models

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v5/chat/completions/models",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "model_vendor": model_vendor,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    completion_models_params.CompletionModelsParams,
                ),
            ),
            cast_to=CompletionModelsResponse,
        )


class CompletionsResourceWithRawResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_raw_response_wrapper(
            completions.create,
        )
        self.models = to_raw_response_wrapper(
            completions.models,
        )


class AsyncCompletionsResourceWithRawResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_raw_response_wrapper(
            completions.create,
        )
        self.models = async_to_raw_response_wrapper(
            completions.models,
        )


class CompletionsResourceWithStreamingResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_streamed_response_wrapper(
            completions.create,
        )
        self.models = to_streamed_response_wrapper(
            completions.models,
        )


class AsyncCompletionsResourceWithStreamingResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_streamed_response_wrapper(
            completions.create,
        )
        self.models = async_to_streamed_response_wrapper(
            completions.models,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["CompletionCreateParamsBase", "CompletionCreateParamsNonStreaming", "CompletionCreateParamsStreaming"]


class CompletionCreateParamsBase(TypedDict, total=False):
    model: Required[str]
    """model specified as `model_vendor/model`, for example `openai/gpt-4o`"""

    prompt: Required[Union[str, SequenceNotStr[str]]]
    """The prompt to generate completions for, encoded as a string"""

    best_of: int
    """Generates best_of completions server-side and returns the best one.

    Must be greater than n when used together.
    """

    echo: bool
    """Echo back the prompt in addition to the completion"""

    frequency_penalty: float
    """Number between -2.0 and 2.0.

    Positive values penalize new tokens based on their existing frequency in the
    text.
    """

    logit_bias: Dict[str, int]
    """Modify the likelihood of specified tokens appearing in the completion.

    Maps tokens to bias values from -100 to 100.
    """

    logprobs: int
    """Include log probabilities of the most likely tokens. Maximum value is 5."""

    max_tokens: int
    """The maximum number of tokens that can be generated in the completion."""

    n: int
    """How many completions to generate for each prompt."""

    presence_penalty: float
    """Number between -2.0 and 2.0.

    Positive values penalize new tokens based on their presence in the text so far.
    """

    seed: int
    """If specified, attempts to generate deterministic samples.

    Determinism is not guaranteed.
    """

    stop: Union[str, SequenceNotStr[str]]
    """Up to 4 sequences where the API will stop generating further tokens."""

    stream_options: Dict[str, object]
    """Options for streaming response. Only set this when stream is True."""

    suffix: str
    """The suffix that comes after a completion of inserted text.

    Only supported for gpt-3.5-turbo-instruct.
    """

    temperature: float
    """Sampling temperature between 0 and 2.

    Higher values make output more random, lower more focused.
    """

    top_p: float
    """Alternative to temperature.

    Consider only tokens with top_p probability mass. Range 0-1.
    """

    user: str
    """
    A unique identifier representing your end-user, which can help OpenAI monitor
    and detect abuse.
    """


class CompletionCreateParamsNonStreaming(CompletionCreateParamsBase, total=False):
    stream: Literal[False]
    """Whether to stream back partial progress.

    If set, tokens will be sent as data-only server-sent events.
    """


class CompletionCreateParamsStreaming(CompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """Whether to stream back partial progress.

    If set, tokens will be sent as data-only server-sent events.
    """


CompletionCreateParams = Union[CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Any, List, Generic, TypeVar, Optional, cast
from typing_extensions import Protocol, override, runtime_checkable

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["SyncCursorPage", "AsyncCursorPage", "SyncAPIListPage", "AsyncAPIListPage"]

_T = TypeVar("_T")


@runtime_checkable
class CursorPageItem(Protocol):
    id: str


class SyncCursorPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    has_more: Optional[bool] = None
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def has_next_page(self) -> bool:
        has_more = self.has_more
        if has_more is not None and has_more is False:
            return False

        return super().has_next_page()

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("ending_before", False)

        items = self.items
        if not items:
            return None

        if is_forwards:
            item = cast(Any, items[-1])
            if not isinstance(item, CursorPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"starting_after": item.id})
        else:
            item = cast(Any, self.items[0])
            if not isinstance(item, CursorPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"ending_before": item.id})


class AsyncCursorPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    has_more: Optional[bool] = None
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def has_next_page(self) -> bool:
        has_more = self.has_more
        if has_more is not None and has_more is False:
            return False

        return super().has_next_page()

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("ending_before", False)

        items = self.items
        if not items:
            return None

        if is_forwards:
            item = cast(Any, items[-1])
            if not isinstance(item, CursorPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"starting_after": item.id})
        else:
            item = cast(Any, self.items[0])
            if not isinstance(item, CursorPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"ending_before": item.id})


class SyncAPIListPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None


class AsyncAPIListPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None



"""
A simple example demonstrating how to trace an OpenAI Agent run.

To run this example:
1. Install the necessary packages:
   pip install openai-agents
2. Fill in your SGP `api_key` and `account_id`.
3. Run the script from your terminal
"""
import os
import asyncio

try:
    from agents import Agent, Runner, function_tool, set_trace_processors  # type: ignore
except ImportError as e:
    raise ImportError("Please install the required packages: pip install openai-agents") from e

import scale_gp_beta.lib.tracing as tracing
from scale_gp_beta import SGPClient
from scale_gp_beta.lib.tracing.integrations import OpenAITracingSGPProcessor


@function_tool  # type: ignore
def get_current_weather(city: str) -> str:
    if "san francisco" in city.lower():
        return "The weather in San Francisco is foggy."
    return f"The weather in {city} is sunny."


async def main() -> None:
    weather_agent = Agent(  # type: ignore
        name="Weather Agent",
        instructions="You are a helpful assistant that can provide weather information.",
        model="gpt-4-turbo",
        tools=[get_current_weather],
    )

    result = await Runner.run(weather_agent, "What's the weather like in San Francisco?")  # type: ignore
    print("Final Output:", result.final_output)  # type: ignore


if __name__ == "__main__":
    api_key = "XXX"
    account_id = "XXX"
    os.environ["OPENAI_API_KEY"] = "XXX"

    tracing.init(SGPClient(api_key=api_key, account_id=account_id), disabled=False)

    sgp_processor = OpenAITracingSGPProcessor()
    set_trace_processors([sgp_processor])
    asyncio.run(main())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, Iterable, cast
from typing_extensions import Literal, overload

import httpx

from ..types import question_list_params, question_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.question import Question

__all__ = ["QuestionsResource", "AsyncQuestionsResource"]


class QuestionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QuestionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return QuestionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QuestionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return QuestionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        configuration: question_create_params.CategoricalQuestionRequestConfiguration,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["categorical"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        configuration: question_create_params.RatingQuestionRequestConfiguration,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["rating"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        configuration: question_create_params.NumberQuestionRequestConfiguration | Omit = omit,
        question_type: Literal["number"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        configuration: question_create_params.FreeTextQuestionRequestConfiguration | Omit = omit,
        question_type: Literal["free_text"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        configuration: question_create_params.FormQuestionRequestConfiguration,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["form"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        configuration: question_create_params.TimestampQuestionRequestConfiguration | Omit = omit,
        question_type: Literal["timestamp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["configuration", "name", "prompt"], ["name", "prompt"])
    def create(
        self,
        *,
        configuration: question_create_params.CategoricalQuestionRequestConfiguration
        | question_create_params.RatingQuestionRequestConfiguration
        | question_create_params.NumberQuestionRequestConfiguration
        | question_create_params.FreeTextQuestionRequestConfiguration
        | question_create_params.FormQuestionRequestConfiguration
        | question_create_params.TimestampQuestionRequestConfiguration
        | Omit = omit,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["categorical"]
        | Literal["rating"]
        | Literal["number"]
        | Literal["free_text"]
        | Literal["form"]
        | Literal["timestamp"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        return cast(
            Question,
            self._post(
                "/v5/questions",
                body=maybe_transform(
                    {
                        "configuration": configuration,
                        "name": name,
                        "prompt": prompt,
                        "conditions": conditions,
                        "question_type": question_type,
                    },
                    question_create_params.QuestionCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, Question),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        question_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Get Question

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_id:
            raise ValueError(f"Expected a non-empty value for `question_id` but received {question_id!r}")
        return cast(
            Question,
            self._get(
                f"/v5/questions/{question_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, Question),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[Question]:
        """
        List Questions

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/questions",
            page=SyncCursorPage[Question],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    question_list_params.QuestionListParams,
                ),
            ),
            model=cast(Any, Question),  # Union types cannot be passed in as arguments in the type system
        )


class AsyncQuestionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQuestionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncQuestionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQuestionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncQuestionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        configuration: question_create_params.CategoricalQuestionRequestConfiguration,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["categorical"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        configuration: question_create_params.RatingQuestionRequestConfiguration,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["rating"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        configuration: question_create_params.NumberQuestionRequestConfiguration | Omit = omit,
        question_type: Literal["number"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        configuration: question_create_params.FreeTextQuestionRequestConfiguration | Omit = omit,
        question_type: Literal["free_text"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        configuration: question_create_params.FormQuestionRequestConfiguration,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["form"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        configuration: question_create_params.TimestampQuestionRequestConfiguration | Omit = omit,
        question_type: Literal["timestamp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Create Question

        Args:
          prompt: user-facing question prompt

          conditions: Conditions for the question to be shown

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["configuration", "name", "prompt"], ["name", "prompt"])
    async def create(
        self,
        *,
        configuration: question_create_params.CategoricalQuestionRequestConfiguration
        | question_create_params.RatingQuestionRequestConfiguration
        | question_create_params.NumberQuestionRequestConfiguration
        | question_create_params.FreeTextQuestionRequestConfiguration
        | question_create_params.FormQuestionRequestConfiguration
        | question_create_params.TimestampQuestionRequestConfiguration
        | Omit = omit,
        name: str,
        prompt: str,
        conditions: Iterable[Dict[str, object]] | Omit = omit,
        question_type: Literal["categorical"]
        | Literal["rating"]
        | Literal["number"]
        | Literal["free_text"]
        | Literal["form"]
        | Literal["timestamp"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        return cast(
            Question,
            await self._post(
                "/v5/questions",
                body=await async_maybe_transform(
                    {
                        "configuration": configuration,
                        "name": name,
                        "prompt": prompt,
                        "conditions": conditions,
                        "question_type": question_type,
                    },
                    question_create_params.QuestionCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, Question),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        question_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Question:
        """
        Get Question

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_id:
            raise ValueError(f"Expected a non-empty value for `question_id` but received {question_id!r}")
        return cast(
            Question,
            await self._get(
                f"/v5/questions/{question_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, Question),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Question, AsyncCursorPage[Question]]:
        """
        List Questions

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/questions",
            page=AsyncCursorPage[Question],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    question_list_params.QuestionListParams,
                ),
            ),
            model=cast(Any, Question),  # Union types cannot be passed in as arguments in the type system
        )


class QuestionsResourceWithRawResponse:
    def __init__(self, questions: QuestionsResource) -> None:
        self._questions = questions

        self.create = to_raw_response_wrapper(
            questions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            questions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            questions.list,
        )


class AsyncQuestionsResourceWithRawResponse:
    def __init__(self, questions: AsyncQuestionsResource) -> None:
        self._questions = questions

        self.create = async_to_raw_response_wrapper(
            questions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            questions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            questions.list,
        )


class QuestionsResourceWithStreamingResponse:
    def __init__(self, questions: QuestionsResource) -> None:
        self._questions = questions

        self.create = to_streamed_response_wrapper(
            questions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            questions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            questions.list,
        )


class AsyncQuestionsResourceWithStreamingResponse:
    def __init__(self, questions: AsyncQuestionsResource) -> None:
        self._questions = questions

        self.create = async_to_streamed_response_wrapper(
            questions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            questions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            questions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["VectorStoreVectorsParams"]


class VectorStoreVectorsParams(TypedDict, total=False):
    cursor: str
    """Pagination cursor"""

    filter: str
    """Metadata filter expression (JSON)"""

    include_vectors: bool
    """Include embedding vectors"""

    limit: int
    """Vectors per page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types import (
    InferenceModel,
    ModelDeleteResponse,
)
from scale_gp_beta.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestModels:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: SGPClient) -> None:
        model = client.models.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                },
                "model_infra": {},
            },
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: SGPClient) -> None:
        model = client.models.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                    "env_vars": {"foo": "bar"},
                    "healthcheck_route": "healthcheck_route",
                    "predict_route": "predict_route",
                    "readiness_delay": 0,
                    "request_schema": {"foo": "bar"},
                    "response_schema": {"foo": "bar"},
                    "streaming_command": ["string"],
                    "streaming_predict_route": "streaming_predict_route",
                },
                "model_infra": {
                    "cpus": "string",
                    "endpoint_type": "async",
                    "gpu_type": "nvidia-tesla-t4",
                    "gpus": 0,
                    "high_priority": True,
                    "labels": {"foo": "string"},
                    "max_workers": 0,
                    "memory": "memory",
                    "min_workers": 0,
                    "per_worker": 0,
                    "public_inference": True,
                    "storage": "storage",
                },
            },
            model_metadata={"foo": "bar"},
            model_type="generic",
            model_vendor="launch",
            on_conflict="error",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                },
                "model_infra": {},
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                },
                "model_infra": {},
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_2(self, client: SGPClient) -> None:
        model = client.models.create(
            name="name",
            vendor_configuration={"model": "model"},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: SGPClient) -> None:
        model = client.models.create(
            name="name",
            vendor_configuration={
                "model": "model",
                "checkpoint_path": "checkpoint_path",
                "cpus": 0,
                "default_callback_url": "default_callback_url",
                "endpoint_type": "endpoint_type",
                "gpu_type": "gpu_type",
                "gpus": 0,
                "high_priority": True,
                "inference_framework": "inference_framework",
                "inference_framework_image_tag": "inference_framework_image_tag",
                "labels": {"foo": "string"},
                "max_workers": 0,
                "memory": "memory",
                "min_workers": 0,
                "nodes_per_worker": 0,
                "num_shards": 0,
                "per_worker": 0,
                "post_inference_hooks": ["string"],
                "public_inference": True,
                "quantize": "quantize",
                "source": "source",
                "storage": "storage",
            },
            model_metadata={"foo": "bar"},
            model_type="chat_completion",
            model_vendor="llmengine",
            on_conflict="error",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_raw_response_create_overload_2(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.create(
            name="name",
            vendor_configuration={"model": "model"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_2(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.create(
            name="name",
            vendor_configuration={"model": "model"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        model = client.models.retrieve(
            "model_id",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.retrieve(
            "model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.retrieve(
            "model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update_overload_1(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_method_update_with_all_params_overload_1(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
            model_metadata={"foo": "bar"},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_raw_response_update_overload_1(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.update(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_streaming_response_update_overload_1(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.update(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_overload_1(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.update(
                model_id="",
            )

    @parametrize
    def test_method_update_overload_2(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
            vendor_configuration={},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_method_update_with_all_params_overload_2(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "env_vars": {"foo": "bar"},
                    "healthcheck_route": "healthcheck_route",
                    "predict_route": "predict_route",
                    "readiness_delay": 0,
                    "registry": "registry",
                    "repository": "repository",
                    "request_schema": {"foo": "bar"},
                    "response_schema": {"foo": "bar"},
                    "streaming_command": ["string"],
                    "streaming_predict_route": "streaming_predict_route",
                    "tag": "tag",
                },
                "model_infra": {
                    "cpus": "string",
                    "endpoint_type": "async",
                    "gpu_type": "nvidia-tesla-t4",
                    "gpus": 0,
                    "high_priority": True,
                    "labels": {"foo": "string"},
                    "max_workers": 0,
                    "memory": "memory",
                    "min_workers": 0,
                    "per_worker": 0,
                    "public_inference": True,
                    "storage": "storage",
                },
            },
            model_metadata={"foo": "bar"},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_raw_response_update_overload_2(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.update(
            model_id="model_id",
            vendor_configuration={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_streaming_response_update_overload_2(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.update(
            model_id="model_id",
            vendor_configuration={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_overload_2(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.update(
                model_id="",
                vendor_configuration={},
            )

    @parametrize
    def test_method_update_overload_3(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
            name="name",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_method_update_with_all_params_overload_3(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
            name="name",
            on_conflict="error",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_raw_response_update_overload_3(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.update(
            model_id="model_id",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    def test_streaming_response_update_overload_3(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.update(
            model_id="model_id",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_overload_3(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.update(
                model_id="",
                name="name",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        model = client.models.list()
        assert_matches_type(SyncCursorPage[InferenceModel], model, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        model = client.models.list(
            ending_before="ending_before",
            limit=1,
            model_vendor="openai",
            name="name",
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(SyncCursorPage[InferenceModel], model, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(SyncCursorPage[InferenceModel], model, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(SyncCursorPage[InferenceModel], model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        model = client.models.delete(
            "model_id",
        )
        assert_matches_type(ModelDeleteResponse, model, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.delete(
            "model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(ModelDeleteResponse, model, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.delete(
            "model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(ModelDeleteResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.delete(
                "",
            )


class TestAsyncModels:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                },
                "model_infra": {},
            },
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                    "env_vars": {"foo": "bar"},
                    "healthcheck_route": "healthcheck_route",
                    "predict_route": "predict_route",
                    "readiness_delay": 0,
                    "request_schema": {"foo": "bar"},
                    "response_schema": {"foo": "bar"},
                    "streaming_command": ["string"],
                    "streaming_predict_route": "streaming_predict_route",
                },
                "model_infra": {
                    "cpus": "string",
                    "endpoint_type": "async",
                    "gpu_type": "nvidia-tesla-t4",
                    "gpus": 0,
                    "high_priority": True,
                    "labels": {"foo": "string"},
                    "max_workers": 0,
                    "memory": "memory",
                    "min_workers": 0,
                    "per_worker": 0,
                    "public_inference": True,
                    "storage": "storage",
                },
            },
            model_metadata={"foo": "bar"},
            model_type="generic",
            model_vendor="launch",
            on_conflict="error",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                },
                "model_infra": {},
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.create(
            name="name",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "registry": "registry",
                    "repository": "repository",
                    "tag": "tag",
                },
                "model_infra": {},
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.create(
            name="name",
            vendor_configuration={"model": "model"},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.create(
            name="name",
            vendor_configuration={
                "model": "model",
                "checkpoint_path": "checkpoint_path",
                "cpus": 0,
                "default_callback_url": "default_callback_url",
                "endpoint_type": "endpoint_type",
                "gpu_type": "gpu_type",
                "gpus": 0,
                "high_priority": True,
                "inference_framework": "inference_framework",
                "inference_framework_image_tag": "inference_framework_image_tag",
                "labels": {"foo": "string"},
                "max_workers": 0,
                "memory": "memory",
                "min_workers": 0,
                "nodes_per_worker": 0,
                "num_shards": 0,
                "per_worker": 0,
                "post_inference_hooks": ["string"],
                "public_inference": True,
                "quantize": "quantize",
                "source": "source",
                "storage": "storage",
            },
            model_metadata={"foo": "bar"},
            model_type="chat_completion",
            model_vendor="llmengine",
            on_conflict="error",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.create(
            name="name",
            vendor_configuration={"model": "model"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.create(
            name="name",
            vendor_configuration={"model": "model"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.retrieve(
            "model_id",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.retrieve(
            "model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.retrieve(
            "model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_method_update_with_all_params_overload_1(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            model_metadata={"foo": "bar"},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_raw_response_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.update(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_streaming_response_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.update(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_overload_1(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.update(
                model_id="",
            )

    @parametrize
    async def test_method_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            vendor_configuration={},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_method_update_with_all_params_overload_2(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            vendor_configuration={
                "model_image": {
                    "command": ["string"],
                    "env_vars": {"foo": "bar"},
                    "healthcheck_route": "healthcheck_route",
                    "predict_route": "predict_route",
                    "readiness_delay": 0,
                    "registry": "registry",
                    "repository": "repository",
                    "request_schema": {"foo": "bar"},
                    "response_schema": {"foo": "bar"},
                    "streaming_command": ["string"],
                    "streaming_predict_route": "streaming_predict_route",
                    "tag": "tag",
                },
                "model_infra": {
                    "cpus": "string",
                    "endpoint_type": "async",
                    "gpu_type": "nvidia-tesla-t4",
                    "gpus": 0,
                    "high_priority": True,
                    "labels": {"foo": "string"},
                    "max_workers": 0,
                    "memory": "memory",
                    "min_workers": 0,
                    "per_worker": 0,
                    "public_inference": True,
                    "storage": "storage",
                },
            },
            model_metadata={"foo": "bar"},
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_raw_response_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.update(
            model_id="model_id",
            vendor_configuration={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_streaming_response_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.update(
            model_id="model_id",
            vendor_configuration={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_overload_2(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.update(
                model_id="",
                vendor_configuration={},
            )

    @parametrize
    async def test_method_update_overload_3(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            name="name",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_method_update_with_all_params_overload_3(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            name="name",
            on_conflict="error",
        )
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_raw_response_update_overload_3(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.update(
            model_id="model_id",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(InferenceModel, model, path=["response"])

    @parametrize
    async def test_streaming_response_update_overload_3(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.update(
            model_id="model_id",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(InferenceModel, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_overload_3(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.update(
                model_id="",
                name="name",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.list()
        assert_matches_type(AsyncCursorPage[InferenceModel], model, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.list(
            ending_before="ending_before",
            limit=1,
            model_vendor="openai",
            name="name",
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(AsyncCursorPage[InferenceModel], model, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(AsyncCursorPage[InferenceModel], model, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(AsyncCursorPage[InferenceModel], model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.delete(
            "model_id",
        )
        assert_matches_type(ModelDeleteResponse, model, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.delete(
            "model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(ModelDeleteResponse, model, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.delete(
            "model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(ModelDeleteResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.delete(
                "",
            )



import time
import queue
import atexit
import logging
import threading
from typing import TYPE_CHECKING, Optional

from scale_gp_beta import SGPClient, SGPClientError

from .util import configure, is_disabled
from .trace_exporter import TraceExporter

if TYPE_CHECKING:
    import httpx

    from .span import Span
    from .trace import Trace

# configurable by env vars?
DEFAULT_MAX_QUEUE_SIZE = 4_000
DEFAULT_TRIGGER_QUEUE_SIZE = 200
DEFAULT_TRIGGER_CADENCE = 4.0
DEFAULT_MAX_BATCH_SIZE = 50
DEFAULT_RETRIES = 4

WORKER_SLEEP_SECONDS = 0.1

log: logging.Logger = logging.getLogger(__name__)


class TraceQueueManager:
    """Manage trace and spans queue
    Store spans in-memory until the threshold has been reached then flush to server.

    Optionally provide a client, if unprovided, we will attempt to create a default client.
    """

    def __init__(
        self,
        client: Optional[SGPClient] = None,
        max_queue_size: int = DEFAULT_MAX_QUEUE_SIZE,
        max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
        trigger_queue_size: int = DEFAULT_TRIGGER_QUEUE_SIZE,
        trigger_cadence: float = DEFAULT_TRIGGER_CADENCE,
        retries: int = DEFAULT_RETRIES,
        worker_enabled: Optional[bool] = None,
    ):
        self._client = client
        self.register_client(client) if client else None
        self._attempted_local_client_creation = False
        self._trigger_queue_size = trigger_queue_size
        self._trigger_cadence = trigger_cadence

        self._reset_trigger_time()

        self._exporter = TraceExporter(max_batch_size, retries)

        self._shutdown_event = threading.Event()
        self._queue: queue.Queue[Span] = queue.Queue(maxsize=max_queue_size)

        self._worker_enabled = worker_enabled if worker_enabled is not None else not is_disabled()

        if self._worker_enabled:
            self._worker = threading.Thread(daemon=True, target=self._run)
            self._worker.start()

            # ensure the thread joins on exit
            atexit.register(self.shutdown)

    def register_client(self, client: SGPClient) -> None:
        log.info("Registering client")
        self._client = client

        original_prepare_request = self._client._prepare_request

        def custom_prepare_request(request: "httpx.Request") -> None:
            original_prepare_request(request)

            # TODO: Hook logic here, we should check to see if we are in the scope of a span, if so we should inject
            # appropriate headers into the request
            # current_span = Scope.get_current_span()

        self._client._prepare_request = custom_prepare_request  # type: ignore

    def shutdown(self, timeout: Optional[float] = None) -> None:
        if not self._worker_enabled:
            log.debug("No worker to shutdown")
            return
        log.info(f"Shutting down trace queue manager, joining worker thread with timeout {timeout}")
        self._shutdown_event.set()
        self._worker.join(timeout=timeout)
        log.info("Shutdown complete")

    def report_span_start(self, span: "Span") -> None:
        # TODO: support making this optional. Current backend requires us to send span starts
        self.enqueue(span)

    def report_span_end(self, span: "Span") -> None:
        self.enqueue(span)

    def report_trace_start(self, trace: "Trace") -> None:
        pass

    def report_trace_end(self, trace: "Trace") -> None:
        pass

    def flush_queue(self) -> None:
        self._export()

    def enqueue(self, span: "Span") -> None:
        try:
            # Should this be a deep copy of span instead? Currently is a reference
            self._queue.put_nowait(span)
        except queue.Full:
            log.warning(f"Queue full, ignoring span {span.span_id}")

    def export_now(self, span: "Span") -> None:
        if self.client:
            self._exporter.export_span(self.client, span)

    @property
    def client(self) -> Optional[SGPClient]:
        """
        Use client provided client on init if available, otherwise attempt once to create a default one.
        :return: SGPClient
        """
        if self._client is not None:
            return self._client
        if self._attempted_local_client_creation:
            # Already tried and failed to create a client
            return None

        log.info("Tracing queue manager not initialized, attempting to create a default one")
        try:
            self.register_client(SGPClient())
        except SGPClientError:
            log.warning(
                f"Failed to create SGPClient for tracing queue manager {self}, ignoring traces. Please initialize with a working client."
            )
        finally:
            self._attempted_local_client_creation = True
        return self._client

    def _run(self) -> None:
        # daemon worker loop
        while not self._shutdown_event.is_set():
            current_time = time.time()
            queue_size = self._queue.qsize()

            if queue_size >= self._trigger_queue_size or current_time >= self._next_trigger_time:
                self._export()
                self._reset_trigger_time()
                continue

            time.sleep(WORKER_SLEEP_SECONDS)

        # flush all on shutdown
        self._export()

    def _export(self) -> None:
        if self.client:
            self._exporter.export(self.client, self._queue)

    def _reset_trigger_time(self) -> None:
        self._next_trigger_time = time.time() + self._trigger_cadence


_global_tracing_queue_manager: Optional[TraceQueueManager] = None
_queue_manager_lock = threading.Lock()


def init(client: Optional[SGPClient] = None, disabled: Optional[bool] = None) -> None:
    """Initialize the tracing backend

    Good practice to always include this method call with a valid client at your program entrypoint.

    Tracing will attempt to generate a default client if unprovided.

    :param client: SGPClient
    :param disabled: Set to True to disable tracing. Overrides environment variable ``DISABLE_SCALE_TRACING``
    """
    if disabled is not None:
        configure(disabled=disabled)

    global _global_tracing_queue_manager
    if _global_tracing_queue_manager is not None:
        return

    with _queue_manager_lock:
        if _global_tracing_queue_manager is None:
            _global_tracing_queue_manager = TraceQueueManager(client)


def tracing_queue_manager() -> TraceQueueManager:
    global _global_tracing_queue_manager
    if _global_tracing_queue_manager is None:
        init(None)

    if _global_tracing_queue_manager is None:
        # should never happen... useful for linting
        raise RuntimeError("Tracing queue manager failed to initialize.")

    return _global_tracing_queue_manager



import logging
from typing import Dict, Optional
from unittest.mock import Mock, MagicMock, patch

import pytest
from pytest import LogCaptureFixture

from scale_gp_beta.lib.tracing.span import Span, BaseSpan
from scale_gp_beta.lib.tracing.trace import Trace, BaseTrace
from scale_gp_beta.lib.tracing.tracing import create_span, create_trace, current_span, current_trace
from scale_gp_beta.lib.tracing.trace_queue_manager import TraceQueueManager


@pytest.fixture
def mock_queue_manager_instance() -> Mock:
    return Mock(spec=TraceQueueManager)

@pytest.fixture
def mock_active_trace_instance() -> Mock:
    trace_mock = Mock(spec=Trace)
    trace_mock.trace_id = "active-trace-id"
    trace_mock.group_id.return_value = "active-group-id"
    return trace_mock

@pytest.fixture
def mock_active_span_instance(mock_active_trace_instance: Mock) -> Mock:
    span_mock = Mock(spec=Span)
    span_mock.span_id = "active-span-id"
    span_mock.trace_id = mock_active_trace_instance.trace_id
    return span_mock


class TestCurrentHelpers:
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_span")
    def test_current_span_calls_scope(self, mock_get_current_span: MagicMock) -> None:
        expected_span = Mock(spec=BaseSpan)
        mock_get_current_span.return_value = expected_span
        assert current_span() == expected_span
        mock_get_current_span.assert_called_once_with()

    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_current_trace_calls_scope(self, mock_get_current_trace: MagicMock) -> None:
        expected_trace = Mock(spec=BaseTrace)
        mock_get_current_trace.return_value = expected_trace
        assert current_trace() == expected_trace
        mock_get_current_trace.assert_called_once_with()


class TestCreateTrace:
    @patch("scale_gp_beta.lib.tracing.tracing.NoOpTrace")
    @patch("scale_gp_beta.lib.tracing.tracing.is_disabled")
    def test_create_trace_when_disabled(
        self, mock_is_disabled: MagicMock, mock_noop_trace_init: MagicMock,
        caplog: LogCaptureFixture
    ) -> None:
        mock_is_disabled.return_value = True
        mock_noop_trace_instance = mock_noop_trace_init.return_value
        trace_id_arg = "custom-trace-id-disabled"
        trace_name_arg = "custom-trace-name-disabled"

        with caplog.at_level(logging.DEBUG):
            trace = create_trace(trace_id=trace_id_arg, name=trace_name_arg)

        assert trace == mock_noop_trace_instance
        mock_is_disabled.assert_called_once_with()
        mock_noop_trace_init.assert_called_once_with(
            name=trace_name_arg,
            trace_id=trace_id_arg,
            span_id=None,
            group_id=None,
            span_type="TRACER",
            metadata={},
            input={},
            output={},
        )
        assert "Tracing is disabled. Not creating a new trace." in caplog.text

    @patch("scale_gp_beta.lib.tracing.tracing.Trace")
    @patch("scale_gp_beta.lib.tracing.tracing.tracing_queue_manager")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_create_trace_when_enabled(
        self,
        mock_get_current_trace: MagicMock,
        mock_tqm_func: MagicMock,
        mock_trace_init: MagicMock,
        mock_queue_manager_instance: Mock,
        caplog: LogCaptureFixture
    ) -> None:
        mock_get_current_trace.return_value = None
        mock_tqm_func.return_value = mock_queue_manager_instance
        mock_trace_instance = mock_trace_init.return_value
        mock_trace_instance.trace_id = "generated-trace-id"
        trace_id_arg = "custom-trace-id-enabled"
        trace_name_arg = "custom-trace-name-enabled"

        with caplog.at_level(logging.DEBUG):
            trace = create_trace(trace_id=trace_id_arg, name=trace_name_arg)

        assert trace == mock_trace_instance
        mock_tqm_func.assert_called_once_with()
        mock_trace_init.assert_called_once_with(
            name=trace_name_arg,
            trace_id=trace_id_arg,
            span_id=None,
            group_id=None,
            queue_manager=mock_queue_manager_instance,
            span_type="TRACER",
            metadata={},
            input={},
            output={},
        )
        assert f"Created new trace: {mock_trace_instance.trace_id}" in caplog.text
        assert "already active" not in caplog.text

    @patch("scale_gp_beta.lib.tracing.tracing.Trace")
    @patch("scale_gp_beta.lib.tracing.tracing.tracing_queue_manager")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_create_trace_when_another_trace_is_active(
        self,
        mock_get_current_trace: MagicMock,
        mock_tqm_func: MagicMock,
        mock_trace_init: MagicMock,
        mock_active_trace_instance: Mock,
        mock_queue_manager_instance: Mock,
        caplog: LogCaptureFixture
    ) -> None:
        mock_get_current_trace.return_value = mock_active_trace_instance
        mock_tqm_func.return_value = mock_queue_manager_instance
        trace_id_arg = "new-trace-while-active"
        trace_name_arg = "new-trace-name-while-active"

        with caplog.at_level(logging.WARNING):
            create_trace(trace_id=trace_id_arg, name=trace_name_arg)

        mock_get_current_trace.assert_called_once_with()
        assert f"Trace with id {mock_active_trace_instance.trace_id} is already active." in caplog.text
        mock_trace_init.assert_called_once()


class TestCreateSpan:

    @patch("scale_gp_beta.lib.tracing.tracing.NoOpSpan")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_span")
    @patch("scale_gp_beta.lib.tracing.tracing.is_disabled")
    def test_create_span_when_disabled(
        self,
        mock_is_disabled: MagicMock,
        mock_get_current_span: MagicMock,
        mock_noop_span_init: MagicMock,
        mock_active_trace_instance: Mock # noqa: ARG002
    ) -> None:
        mock_get_current_span.return_value = None
        mock_is_disabled.return_value = True
        mock_noop_span_instance = mock_noop_span_init.return_value
        span_name = "disabled_span"
        span_id_arg = "custom-span-id-disabled"

        mock_parent_trace = Mock(spec=Trace)
        mock_parent_trace.trace_id = "parent-trace-id-for-disabled"

        span = create_span(name=span_name, span_id=span_id_arg, trace_id=mock_parent_trace.trace_id)

        assert span == mock_noop_span_instance
        mock_is_disabled.assert_called_once_with()
        mock_noop_span_init.assert_called_once_with(
            name=span_name,
            span_id=span_id_arg,
            parent_span_id=None,
            group_id=None,
            trace_id=mock_parent_trace.trace_id,
            input={}, output={}, metadata={},
            span_type="STANDALONE"
        )

    @patch("scale_gp_beta.lib.tracing.tracing.NoOpSpan")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_span")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_create_span_no_parent_no_active_trace(
        self,
        mock_get_current_trace: MagicMock,
        mock_get_current_span: MagicMock,
        mock_noop_span_init: MagicMock,
        caplog: LogCaptureFixture
    ) -> None:
        mock_get_current_trace.return_value = None
        mock_get_current_span.return_value = None
        mock_noop_span_instance = mock_noop_span_init.return_value
        span_name = "orphaned_span"
        span_id_arg = "custom-span-id-orphaned"
        group_id_arg = "custom-group-id-orphaned"

        with caplog.at_level(logging.DEBUG):
            span = create_span(name=span_name, span_id=span_id_arg, group_id=group_id_arg)

        assert span == mock_noop_span_instance
        mock_get_current_trace.assert_called_once_with()
        mock_noop_span_init.assert_called_once_with(
            name=span_name,
            span_id=span_id_arg,
            parent_span_id=None,
            group_id=group_id_arg,
            trace_id=None,
            input={}, output={}, metadata={},
            span_type="STANDALONE"
        )
        assert "Attempting to create a span with no trace" in caplog.text

    @patch("scale_gp_beta.lib.tracing.tracing.Span")
    @patch("scale_gp_beta.lib.tracing.tracing.tracing_queue_manager")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_span")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_create_span_with_explicit_trace_parent(
        self,
        mock_get_current_trace: MagicMock, # noqa: ARG002
        mock_get_current_span: MagicMock,
        mock_tqm_func: MagicMock,
        mock_span_init: MagicMock,
        mock_queue_manager_instance: Mock,
        caplog: LogCaptureFixture,
        mock_active_span_instance: Mock
    ) -> None:
        mock_tqm_func.return_value = mock_queue_manager_instance
        mock_span_instance = mock_span_init.return_value
        mock_span_instance.span_id = "generated-span-id-trace-parent"
        mock_get_current_span.return_value = mock_active_span_instance

        mock_parent_trace = Mock(spec=Trace)
        mock_parent_trace.trace_id = "explicit-parent-trace-id"
        mock_parent_trace.group_id = "explicit-parent-group-id"
        span_name = "span_with_trace_parent"
        span_id_arg = "custom-id-trace-parent"
        input_arg = {"input_key": "input_val"}
        output_arg = {"output_key": "output_val"}
        metadata_arg: Dict[str, Optional[str]] = {"meta_key": "meta_val"}


        with caplog.at_level(logging.DEBUG):
            span = create_span(
                name=span_name,
                span_id=span_id_arg,
                trace_id=mock_parent_trace.trace_id,
                group_id=mock_parent_trace.group_id,
                input=input_arg,
                output=output_arg,
                metadata=metadata_arg
            )

        assert span == mock_span_instance
        mock_tqm_func.assert_called_once_with()
        mock_span_init.assert_called_once_with(
            name=span_name,
            span_id=span_id_arg,
            parent_span_id=mock_active_span_instance.span_id,
            group_id=mock_parent_trace.group_id,
            trace_id=mock_parent_trace.trace_id,
            input=input_arg,
            output=output_arg,
            metadata=metadata_arg,
            queue_manager=mock_queue_manager_instance,
            span_type="STANDALONE"
        )
        assert f"Created new span: {mock_span_instance.span_id}" in caplog.text

    @pytest.mark.skip(reason="Currently do not support searching for parent span and finding current trace")
    @patch("scale_gp_beta.lib.tracing.tracing.Span")
    @patch("scale_gp_beta.lib.tracing.tracing.tracing_queue_manager")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_span")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_create_span_with_explicit_span_parent(
        self,
        mock_get_current_trace: MagicMock, # noqa: ARG002
        mock_get_current_span: MagicMock, # noqa: ARG002
        mock_tqm_func: MagicMock,
        mock_span_init: MagicMock,
        mock_queue_manager_instance: Mock,
    ) -> None:
        mock_tqm_func.return_value = mock_queue_manager_instance

        mock_parent_s = Mock(spec=Span)
        mock_parent_s.group_id = "explicit-parent-group-id"
        mock_parent_s.trace_id = "parent-span-trace-id"
        mock_parent_s.span_id = "parent-span-actual-id"
        span_name = "span_with_span_parent"

        create_span(name=span_name, parent_id=mock_parent_s.span_id, group_id=mock_parent_s.group_id)

        # NOTE: this will grab the trace_id from current context and not mock_parent_s.trace_id
        mock_span_init.assert_called_once_with(
            name=span_name,
            span_id=None,
            parent_span_id=mock_parent_s.span_id,
            trace_id=mock_parent_s.trace_id,
            group_id=mock_parent_s.group_id,
            input={}, output={}, metadata={},
            queue_manager=mock_queue_manager_instance,
            span_type="STANDALONE"
        )

    @patch("scale_gp_beta.lib.tracing.tracing.Span")
    @patch("scale_gp_beta.lib.tracing.tracing.tracing_queue_manager")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_span")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_create_span_no_explicit_parent_uses_scope_trace(
        self,
        mock_get_current_trace: MagicMock,
        mock_get_current_span: MagicMock,
        mock_tqm_func: MagicMock,
        mock_span_init: MagicMock,
        mock_active_trace_instance: Mock,
        mock_queue_manager_instance: Mock,
    ) -> None:
        mock_get_current_trace.return_value = mock_active_trace_instance
        mock_get_current_span.return_value = None
        mock_tqm_func.return_value = mock_queue_manager_instance

        create_span(name="span_uses_scope_trace")

        mock_get_current_trace.assert_called_once_with()
        mock_get_current_span.assert_called_once_with()
        mock_span_init.assert_called_once_with(
            name="span_uses_scope_trace",
            span_id=None,
            parent_span_id=None,
            group_id=mock_active_trace_instance.group_id,
            trace_id=mock_active_trace_instance.trace_id,
            input={}, output={}, metadata={},
            queue_manager=mock_queue_manager_instance,
            span_type="STANDALONE"
        )

    @patch("scale_gp_beta.lib.tracing.tracing.Span")
    @patch("scale_gp_beta.lib.tracing.tracing.tracing_queue_manager")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_span")
    @patch("scale_gp_beta.lib.tracing.tracing.Scope.get_current_trace")
    def test_create_span_no_explicit_parent_uses_scope_span(
        self,
        mock_get_current_trace: MagicMock,
        mock_get_current_span: MagicMock,
        mock_tqm_func: MagicMock,
        mock_span_init: MagicMock,
        mock_active_trace_instance: Mock,
        mock_active_span_instance: Mock,
        mock_queue_manager_instance: Mock,
    ) -> None:
        mock_get_current_trace.return_value = mock_active_trace_instance
        mock_get_current_span.return_value = mock_active_span_instance
        mock_tqm_func.return_value = mock_queue_manager_instance

        create_span(name="span_uses_scope_span")

        mock_get_current_trace.assert_called_once_with()
        mock_get_current_span.assert_called_once_with()
        mock_span_init.assert_called_once_with(
            name="span_uses_scope_span",
            span_id=None,
            parent_span_id=mock_active_span_instance.span_id,
            trace_id=mock_active_trace_instance.trace_id,
            group_id=mock_active_trace_instance.group_id,
            input={}, output={}, metadata={},
            queue_manager=mock_queue_manager_instance,
            span_type="STANDALONE"
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.identity import Identity

__all__ = ["Dataset"]


class Dataset(BaseModel):
    id: str

    created_at: datetime

    created_by: Identity
    """The identity that created the entity."""

    current_version_num: int

    name: str

    tags: List[str]
    """The tags associated with the entity"""

    archived_at: Optional[datetime] = None

    description: Optional[str] = None

    object: Optional[Literal["dataset"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from . import container, evaluation
from .. import _compat
from .file import File as File
from .span import Span as Span
from .shared import Identity as Identity
from .dataset import Dataset as Dataset
from .question import Question as Question
from .response import Response as Response
from .component import Component as Component
from .container import Container as Container
from .span_type import SpanType as SpanType
from .completion import Completion as Completion
from .credential import Credential as Credential
from .evaluation import Evaluation as Evaluation
from .span_status import SpanStatus as SpanStatus
from .dataset_item import DatasetItem as DatasetItem
from .item_locator import ItemLocator as ItemLocator
from .stream_chunk import StreamChunk as StreamChunk
from .vector_store import VectorStore as VectorStore
from .approval_status import ApprovalStatus as ApprovalStatus
from .assessment_type import AssessmentType as AssessmentType
from .component_param import ComponentParam as ComponentParam
from .container_param import ContainerParam as ContainerParam
from .evaluation_item import EvaluationItem as EvaluationItem
from .evaluation_task import EvaluationTask as EvaluationTask
from .inference_model import InferenceModel as InferenceModel
from .span_assessment import SpanAssessment as SpanAssessment
from .file_list_params import FileListParams as FileListParams
from .build_list_params import BuildListParams as BuildListParams
from .credential_secret import CredentialSecret as CredentialSecret
from .model_list_params import ModelListParams as ModelListParams
from .span_batch_params import SpanBatchParams as SpanBatchParams
from .file_create_params import FileCreateParams as FileCreateParams
from .file_update_params import FileUpdateParams as FileUpdateParams
from .inference_response import InferenceResponse as InferenceResponse
from .span_create_params import SpanCreateParams as SpanCreateParams
from .span_search_params import SpanSearchParams as SpanSearchParams
from .span_update_params import SpanUpdateParams as SpanUpdateParams
from .build_create_params import BuildCreateParams as BuildCreateParams
from .build_list_response import BuildListResponse as BuildListResponse
from .dataset_list_params import DatasetListParams as DatasetListParams
from .model_create_params import ModelCreateParams as ModelCreateParams
from .model_update_params import ModelUpdateParams as ModelUpdateParams
from .span_batch_response import SpanBatchResponse as SpanBatchResponse
from .evaluation_dashboard import EvaluationDashboard as EvaluationDashboard
from .file_delete_response import FileDeleteResponse as FileDeleteResponse
from .question_list_params import QuestionListParams as QuestionListParams
from .build_cancel_response import BuildCancelResponse as BuildCancelResponse
from .build_create_response import BuildCreateResponse as BuildCreateResponse
from .dataset_create_params import DatasetCreateParams as DatasetCreateParams
from .dataset_update_params import DatasetUpdateParams as DatasetUpdateParams
from .evaluation_task_param import EvaluationTaskParam as EvaluationTaskParam
from .item_locator_template import ItemLocatorTemplate as ItemLocatorTemplate
from .model_delete_response import ModelDeleteResponse as ModelDeleteResponse
from .credential_list_params import CredentialListParams as CredentialListParams
from .evaluation_list_params import EvaluationListParams as EvaluationListParams
from .question_create_params import QuestionCreateParams as QuestionCreateParams
from .response_create_params import ResponseCreateParams as ResponseCreateParams
from .build_retrieve_response import BuildRetrieveResponse as BuildRetrieveResponse
from .dataset_delete_response import DatasetDeleteResponse as DatasetDeleteResponse
from .dataset_retrieve_params import DatasetRetrieveParams as DatasetRetrieveParams
from .inference_create_params import InferenceCreateParams as InferenceCreateParams
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .credential_create_params import CredentialCreateParams as CredentialCreateParams
from .credential_update_params import CredentialUpdateParams as CredentialUpdateParams
from .dataset_item_list_params import DatasetItemListParams as DatasetItemListParams
from .evaluation_create_params import EvaluationCreateParams as EvaluationCreateParams
from .evaluation_update_params import EvaluationUpdateParams as EvaluationUpdateParams
from .inference_response_chunk import InferenceResponseChunk as InferenceResponseChunk
from .response_create_response import ResponseCreateResponse as ResponseCreateResponse
from .span_upsert_batch_params import SpanUpsertBatchParams as SpanUpsertBatchParams
from .vector_store_list_params import VectorStoreListParams as VectorStoreListParams
from .inference_create_response import InferenceCreateResponse as InferenceCreateResponse
from .vector_store_count_params import VectorStoreCountParams as VectorStoreCountParams
from .vector_store_query_params import VectorStoreQueryParams as VectorStoreQueryParams
from .credential_delete_response import CredentialDeleteResponse as CredentialDeleteResponse
from .dataset_item_update_params import DatasetItemUpdateParams as DatasetItemUpdateParams
from .evaluation_retrieve_params import EvaluationRetrieveParams as EvaluationRetrieveParams
from .evaluation_schema_response import EvaluationSchemaResponse as EvaluationSchemaResponse
from .span_upsert_batch_response import SpanUpsertBatchResponse as SpanUpsertBatchResponse
from .vector_store_create_params import VectorStoreCreateParams as VectorStoreCreateParams
from .vector_store_delete_params import VectorStoreDeleteParams as VectorStoreDeleteParams
from .vector_store_drop_response import VectorStoreDropResponse as VectorStoreDropResponse
from .vector_store_upsert_params import VectorStoreUpsertParams as VectorStoreUpsertParams
from .evaluation_item_list_params import EvaluationItemListParams as EvaluationItemListParams
from .span_assessment_list_params import SpanAssessmentListParams as SpanAssessmentListParams
from .vector_store_count_response import VectorStoreCountResponse as VectorStoreCountResponse
from .vector_store_query_response import VectorStoreQueryResponse as VectorStoreQueryResponse
from .vector_store_vectors_params import VectorStoreVectorsParams as VectorStoreVectorsParams
from .dataset_item_delete_response import DatasetItemDeleteResponse as DatasetItemDeleteResponse
from .dataset_item_retrieve_params import DatasetItemRetrieveParams as DatasetItemRetrieveParams
from .vector_store_delete_response import VectorStoreDeleteResponse as VectorStoreDeleteResponse
from .vector_store_upsert_response import VectorStoreUpsertResponse as VectorStoreUpsertResponse
from .evaluation_item_export_params import EvaluationItemExportParams as EvaluationItemExportParams
from .file_import_from_cloud_params import FileImportFromCloudParams as FileImportFromCloudParams
from .span_assessment_create_params import SpanAssessmentCreateParams as SpanAssessmentCreateParams
from .span_assessment_update_params import SpanAssessmentUpdateParams as SpanAssessmentUpdateParams
from .vector_store_configure_params import VectorStoreConfigureParams as VectorStoreConfigureParams
from .vector_store_vectors_response import VectorStoreVectorsResponse as VectorStoreVectorsResponse
from .evaluation_item_export_response import EvaluationItemExportResponse as EvaluationItemExportResponse
from .evaluation_item_retrieve_params import EvaluationItemRetrieveParams as EvaluationItemRetrieveParams
from .file_import_from_cloud_response import FileImportFromCloudResponse as FileImportFromCloudResponse
from .span_assessment_delete_response import SpanAssessmentDeleteResponse as SpanAssessmentDeleteResponse
from .dataset_item_batch_create_params import DatasetItemBatchCreateParams as DatasetItemBatchCreateParams
from .evaluation_dashboard_list_params import EvaluationDashboardListParams as EvaluationDashboardListParams
from .evaluation_retrieve_schema_params import EvaluationRetrieveSchemaParams as EvaluationRetrieveSchemaParams
from .dataset_item_batch_create_response import DatasetItemBatchCreateResponse as DatasetItemBatchCreateResponse
from .evaluation_dashboard_create_params import EvaluationDashboardCreateParams as EvaluationDashboardCreateParams
from .evaluation_dashboard_update_params import EvaluationDashboardUpdateParams as EvaluationDashboardUpdateParams
from .evaluation_dashboard_retrieve_params import EvaluationDashboardRetrieveParams as EvaluationDashboardRetrieveParams

# Rebuild cyclical models only after all modules are imported.
# This ensures that, when building the deferred (due to cyclical references) model schema,
# Pydantic can resolve the necessary references.
# See: https://github.com/pydantic/pydantic/issues/11250 for more context.
if _compat.PYDANTIC_V1:
    evaluation.Evaluation.update_forward_refs()  # type: ignore
    container.Container.update_forward_refs()  # type: ignore
else:
    evaluation.Evaluation.model_rebuild(_parent_namespace_depth=0)
    container.Container.model_rebuild(_parent_namespace_depth=0)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "ModelUpdateParams",
    "DefaultModelPatchRequest",
    "ModelConfigurationPatchRequest",
    "ModelConfigurationPatchRequestVendorConfiguration",
    "ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfiguration",
    "ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfigurationModelImage",
    "ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfigurationModelInfra",
    "ModelConfigurationPatchRequestVendorConfigurationPartialLlmEngineVendorConfiguration",
    "SwapNamesModelPatchRequest",
]


class DefaultModelPatchRequest(TypedDict, total=False):
    model_metadata: Dict[str, object]


class ModelConfigurationPatchRequest(TypedDict, total=False):
    vendor_configuration: Required[ModelConfigurationPatchRequestVendorConfiguration]

    model_metadata: Dict[str, object]


class ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfigurationModelImage(
    TypedDict, total=False
):
    command: SequenceNotStr[str]

    env_vars: Dict[str, object]

    healthcheck_route: str

    predict_route: str

    readiness_delay: int

    registry: str

    repository: str

    request_schema: Dict[str, object]

    response_schema: Dict[str, object]

    streaming_command: SequenceNotStr[str]

    streaming_predict_route: str

    tag: str


class ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfigurationModelInfra(
    TypedDict, total=False
):
    cpus: Union[str, int]

    endpoint_type: Literal["async", "sync", "streaming"]

    gpu_type: Literal[
        "nvidia-tesla-t4",
        "nvidia-ampere-a10",
        "nvidia-ampere-a100",
        "nvidia-ampere-a100e",
        "nvidia-hopper-h100",
        "nvidia-hopper-h100-1g20gb",
        "nvidia-hopper-h100-3g40gb",
    ]

    gpus: int

    high_priority: bool

    labels: Dict[str, str]

    max_workers: int

    memory: str

    min_workers: int

    per_worker: int

    public_inference: bool

    storage: str


class ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfiguration(TypedDict, total=False):
    model_image: ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfigurationModelImage

    model_infra: ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfigurationModelInfra


class ModelConfigurationPatchRequestVendorConfigurationPartialLlmEngineVendorConfiguration(TypedDict, total=False):
    checkpoint_path: str

    cpus: int

    default_callback_url: str

    endpoint_type: str

    gpu_type: str

    gpus: int

    high_priority: bool

    inference_framework: str

    inference_framework_image_tag: str

    labels: Dict[str, str]

    max_workers: int

    memory: str

    min_workers: int

    model: str

    nodes_per_worker: int

    num_shards: int

    per_worker: int

    post_inference_hooks: SequenceNotStr[str]

    public_inference: bool

    quantize: str

    source: str

    storage: str


ModelConfigurationPatchRequestVendorConfiguration: TypeAlias = Union[
    ModelConfigurationPatchRequestVendorConfigurationPartialLaunchVendorConfiguration,
    ModelConfigurationPatchRequestVendorConfigurationPartialLlmEngineVendorConfiguration,
]


class SwapNamesModelPatchRequest(TypedDict, total=False):
    name: Required[str]

    on_conflict: Literal["error", "swap"]


ModelUpdateParams: TypeAlias = Union[
    DefaultModelPatchRequest, ModelConfigurationPatchRequest, SwapNamesModelPatchRequest
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["VectorStore"]


class VectorStore(BaseModel):
    """Response model for vector store operations."""

    created_at: datetime
    """Timestamp of creation"""

    embedding_dimensions: int
    """Dimension size of the embeddings"""

    embedding_model: Literal[
        "sentence-transformers/all-MiniLM-L12-v2",
        "sentence-transformers/all-mpnet-base-v2",
        "sentence-transformers/multi-qa-distilbert-cos-v1",
        "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
        "openai/text-embedding-ada-002",
        "openai/text-embedding-3-small",
        "openai/text-embedding-3-large",
        "embed-english-v3.0",
        "embed-english-light-v3.0",
        "embed-multilingual-v3.0",
    ]
    """The embedding model"""

    name: str
    """The name of the vector store"""

    updated_at: datetime
    """Timestamp of last update"""

    indexed_metadata_fields: Optional[Dict[str, Literal["string", "number", "boolean", "object", "list"]]] = None
    """Dictionary mapping metadata field names to their types"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "QuestionCreateParams",
    "CategoricalQuestionRequest",
    "CategoricalQuestionRequestConfiguration",
    "RatingQuestionRequest",
    "RatingQuestionRequestConfiguration",
    "NumberQuestionRequest",
    "NumberQuestionRequestConfiguration",
    "FreeTextQuestionRequest",
    "FreeTextQuestionRequestConfiguration",
    "FormQuestionRequest",
    "FormQuestionRequestConfiguration",
    "TimestampQuestionRequest",
    "TimestampQuestionRequestConfiguration",
]


class CategoricalQuestionRequest(TypedDict, total=False):
    configuration: Required[CategoricalQuestionRequestConfiguration]

    name: Required[str]

    prompt: Required[str]
    """user-facing question prompt"""

    conditions: Iterable[Dict[str, object]]
    """Conditions for the question to be shown"""

    question_type: Literal["categorical"]


class CategoricalQuestionRequestConfiguration(TypedDict, total=False):
    choices: Required[SequenceNotStr[str]]
    """Categorical answer choices (must contain at least one entry)"""

    dropdown: bool
    """Whether the question is displayed as a dropdown in the UI."""

    multi: bool
    """Whether the question allows multiple answers."""


class RatingQuestionRequest(TypedDict, total=False):
    configuration: Required[RatingQuestionRequestConfiguration]

    name: Required[str]

    prompt: Required[str]
    """user-facing question prompt"""

    conditions: Iterable[Dict[str, object]]
    """Conditions for the question to be shown"""

    question_type: Literal["rating"]


class RatingQuestionRequestConfiguration(TypedDict, total=False):
    max_label: Required[str]
    """Label shown for the maximum rating"""

    min_label: Required[str]
    """Label shown for the minimum rating"""

    steps: Required[int]
    """Number of discrete points on the scale (e.g., 5 for a 1–5 scale)"""


class NumberQuestionRequest(TypedDict, total=False):
    name: Required[str]

    prompt: Required[str]
    """user-facing question prompt"""

    conditions: Iterable[Dict[str, object]]
    """Conditions for the question to be shown"""

    configuration: NumberQuestionRequestConfiguration

    question_type: Literal["number"]


class NumberQuestionRequestConfiguration(TypedDict, total=False):
    max: float
    """Maximum value for the number"""

    min: float
    """Minimum value for the number"""


class FreeTextQuestionRequest(TypedDict, total=False):
    name: Required[str]

    prompt: Required[str]
    """user-facing question prompt"""

    conditions: Iterable[Dict[str, object]]
    """Conditions for the question to be shown"""

    configuration: FreeTextQuestionRequestConfiguration

    question_type: Literal["free_text"]


class FreeTextQuestionRequestConfiguration(TypedDict, total=False):
    max_length: int
    """Maximum characters allowed"""

    min_length: int
    """Minimum characters required"""


class FormQuestionRequest(TypedDict, total=False):
    configuration: Required[FormQuestionRequestConfiguration]

    name: Required[str]

    prompt: Required[str]
    """user-facing question prompt"""

    conditions: Iterable[Dict[str, object]]
    """Conditions for the question to be shown"""

    question_type: Literal["form"]


class FormQuestionRequestConfiguration(TypedDict, total=False):
    form_schema: Required[Dict[str, object]]
    """The JSON schema of the desired form object"""


class TimestampQuestionRequest(TypedDict, total=False):
    name: Required[str]

    prompt: Required[str]
    """user-facing question prompt"""

    conditions: Iterable[Dict[str, object]]
    """Conditions for the question to be shown"""

    configuration: TimestampQuestionRequestConfiguration

    question_type: Literal["timestamp"]


class TimestampQuestionRequestConfiguration(TypedDict, total=False):
    multi: bool
    """Whether to allow multiple timestamps"""


QuestionCreateParams: TypeAlias = Union[
    CategoricalQuestionRequest,
    RatingQuestionRequest,
    NumberQuestionRequest,
    FreeTextQuestionRequest,
    FormQuestionRequest,
    TimestampQuestionRequest,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .evaluation_dashboard_widget import EvaluationDashboardWidget

__all__ = ["EvaluationDashboardWidgetResult"]


class EvaluationDashboardWidgetResult(BaseModel):
    id: str
    """Unique identifier of the widget result"""

    account_id: str
    """Account that owns this widget result"""

    computation_status: Literal["pending", "completed", "failed"]
    """Status of the computation"""

    created_at: datetime
    """When the widget result was created"""

    widget_id: str
    """Unique identifier of the widget"""

    computation_job_id: Optional[str] = None
    """Temporal workflow ID or job ID for async computation tracking"""

    computed_at: Optional[datetime] = None
    """Timestamp when computation completed successfully"""

    computed_result: Optional[Dict[str, object]] = None
    """Cached computation results"""

    error_message: Optional[str] = None
    """Error message if computation failed"""

    evaluation_group_id: Optional[str] = None
    """FK to evaluation_groups. Null if result is for a single evaluation."""

    evaluation_id: Optional[str] = None
    """FK to evaluations. Null if result is for an evaluation group."""

    object: Optional[Literal["evaluation_dashboard_widget_result"]] = None

    widget: Optional[EvaluationDashboardWidget] = None
    """Widget that this result is for"""
